import os
from unittest.mock import mock_open, patch

from casa_envoy.launchers import CalibrationLauncher, ImagingLauncher


class TestCalibrationLauncher:
    @patch("subprocess.Popen")
    @patch("os.chdir")
    def test_run(self, mock_os, mock_subprocess):
        CalibrationLauncher("test/PPR.xml", "test/test.json").run()
        assert mock_subprocess.call_count == 1

    def test_setup_environment(self):
        parameters = {
            "useCasa": False,
            "homeForReprocessing": "/home/casa/packages/pipeline/current",
            "rootDirectory": "/tmp/workspaces_tmp/",
            "processingDirectory": "tmpiox5trbp",
        }
        CalibrationLauncher("PPR.xml", "test.json").setup_environment(parameters=parameters)
        assert os.environ.get("SCIPIPE_ROOTDIR") == parameters["rootDirectory"]
        assert os.environ.get("CASA_HOME") == parameters["homeForReprocessing"]
        assert os.environ.get("PPR_FILENAME") == "PPR.xml"

    @patch("builtins.open")
    @patch("glob.glob")
    @patch("os.chdir")
    @patch("os.getcwd")
    def test_check_logs(self, mock_os_cwd, mock_os_dir, mock_glob, mock_open):
        CalibrationLauncher("PPR.xml", "test.json").check_logs(parent_path=".")
        assert mock_os_cwd.call_count == 1
        assert mock_os_dir.call_count == 0
        assert mock_glob.call_count == 1


class TestImagingLauncher:
    @patch("os.chdir")
    def test_run(self, mock_os):
        with patch("subprocess.Popen") as sp:
            ImagingLauncher("test/cmsimage-PPR.xml", "test/image-metadata.json").run()
            assert sp.call_count == 1

    def test_setup_environment(self):
        parameters = {
            "useCasa": False,
            "homeForReprocessing": "/home/casa/packages/pipeline/current",
            "rootDirectory": "/tmp/workspaces_tmp/",
            "processingDirectory": "tmpiox5trbp",
        }
        ppr = "cmsimage-PPR.xml"
        metadata = "image-metadata.json"
        ImagingLauncher(ppr, metadata).setup_environment(parameters=parameters)
        assert os.environ.get("SCIPIPE_ROOTDIR") == parameters["rootDirectory"]
        assert os.environ.get("CASA_HOME") == parameters["homeForReprocessing"]
        assert os.environ.get("PPR_FILENAME") == ppr

    @patch("builtins.open")
    @patch("glob.glob")
    @patch("os.chdir")
    @patch("os.getcwd")
    def test_check_logs(self, mock_os_cwd, mock_os_dir, mock_glob, mock_open):
        ppr = "cmsimage-PPR.xml"
        metadata = "image-metadata.json"
        ImagingLauncher(ppr, metadata).check_logs(parent_path=".")
        assert mock_os_cwd.call_count == 1
        assert mock_os_dir.call_count == 0
        assert mock_glob.call_count == 1
